@@ -3,7 +3,7 @@ Tests for JSON-RPC dispatcher.
 """
 
 import json
-from types import SimpleNamespace
+from types import SimpleNamespace, ModuleType
 
 import pytest
 import mock
@@ -379,6 +379,10 @@ async def test_method_decorator():
     with_context = receives_context(mock.AsyncMock(return_value = 11))
     with_context.__name__ = 'with_context'
     dispatcher.register(with_context)
+    # Test that the name can be overridden using the decorator
+    @dispatcher.register(name = 'overridden_name')
+    async def add(x, y):
+        return x + y
 
     response = await dispatcher.dispatch(
         """
@@ -386,7 +390,8 @@ async def test_method_decorator():
             {"jsonrpc": "2.0", "method": "subtract", "params": [23, 42], "id": "1"},
             {"jsonrpc": "2.0", "method": "no_context_args", "params": [23, 42], "id": "2"},
             {"jsonrpc": "2.0", "method": "no_context_kwargs", "params": {"arg1": 23, "arg2": 42}, "id": "3"},
-            {"jsonrpc": "2.0", "method": "with_context", "params": [23, 42], "id": "4"}
+            {"jsonrpc": "2.0", "method": "with_context", "params": [23, 42], "id": "4"},
+            {"jsonrpc": "2.0", "method": "overridden_name", "params": [23, 42], "id": "5"}
         ]
         """,
         dict(context_key = 'context value')
@@ -395,7 +400,8 @@ async def test_method_decorator():
         {"jsonrpc": "2.0", "result": -19, "id": "1"},
         {"jsonrpc": "2.0", "result": 10, "id": "2"},
         {"jsonrpc": "2.0", "result": 9, "id": "3"},
-        {"jsonrpc": "2.0", "result": 11, "id": "4"}
+        {"jsonrpc": "2.0", "result": 11, "id": "4"},
+        {"jsonrpc": "2.0", "result": 65, "id": "5"},
     ]
     assert json.loads(response) == expected
     no_context_args.assert_called_once_with(23, 42)
@@ -538,11 +544,21 @@ async def test_register_all():
     exclude_group.not_excluded_2 = mock.AsyncMock(return_value = 26)
     exclude_group.excluded = mock.AsyncMock(return_value = 27)
 
+    # Test registering from a module
+    test_module = ModuleType('test_module')
+    test_module.__dict__.update({
+        '__all__': ['in_all_1', 'in_all_2'],
+        'in_all_1': mock.AsyncMock(return_value = 28),
+        'in_all_2': mock.AsyncMock(return_value = 29),
+        'not_in_all': mock.AsyncMock(return_value = 30),
+    })
+
     # Configure the dispatcher
     dispatcher = Dispatcher()
     dispatcher.register_all(method_group, 'grouped')
     dispatcher.register_all(include_group, 'include', only = ['included'])
     dispatcher.register_all(exclude_group, 'exclude', exclude = ['excluded'])
+    dispatcher.register_all(test_module, 'test_module')
 
     # Issue a patch request with a mixture of successful, invalid, errors and notifications
     response = await dispatcher.dispatch(
@@ -556,7 +572,10 @@ async def test_register_all():
             {"jsonrpc": "2.0", "method": "include.not_included", "params": [1,2], "id": "6"},
             {"jsonrpc": "2.0", "method": "exclude.not_excluded_1", "params": [1,2], "id": "7"},
             {"jsonrpc": "2.0", "method": "exclude.not_excluded_2", "params": [1,2], "id": "8"},
-            {"jsonrpc": "2.0", "method": "exclude.excluded", "params": [1,2], "id": "9"}
+            {"jsonrpc": "2.0", "method": "exclude.excluded", "params": [1,2], "id": "9"},
+            {"jsonrpc": "2.0", "method": "test_module.in_all_1", "params": [1,2], "id": "10"},
+            {"jsonrpc": "2.0", "method": "test_module.in_all_2", "params": [1,2], "id": "11"},
+            {"jsonrpc": "2.0", "method": "test_module.not_in_all", "params": [1,2], "id": "12"}
         ]
         """,
         dict(context_key1 = "1")
@@ -603,6 +622,17 @@ async def test_register_all():
             },
             "id": "9"
         },
+        {"jsonrpc": "2.0", "result": 28, "id": "10"},
+        {"jsonrpc": "2.0", "result": 29, "id": "11"},
+        {
+            "jsonrpc": "2.0",
+            "error": {
+                "code": -32601,
+                "message": "Method not found",
+                "data": "no such method: test_module.not_in_all"
+            },
+            "id": "12"
+        },
     ]
     assert json.loads(response) == expected
     method_group.method_1.assert_called_once_with(1, 2)
